from .models import  Ticket
from django.contrib.auth import get_user_model
User = get_user_model()

def get_pending_tickets(request):
    return {
        'pending_tickets': Ticket.objects.filter(status="pending").count(),
    }



from rest_framework import serializers ,exceptions 
from .models import Ticket , TicketMessage
import difflib
from django.utils.translation import gettext as _


class CreateTicketSerializer(serializers.ModelSerializer):
    message = serializers.CharField(required=True)
    class Meta:
        model = Ticket
        fields = ("user","status","title","section","priority","seen_by_admin","seen_by_user","message")
    
    def validate(self, attrs):
        valid = super().validate(attrs)
        valid["status"] = "pending"
        valid["seen_by_user"] = True
        valid["seen_by_admin"] = False
        if not attrs.get("priority"):
            valid["priority"] = "کم"
        if not attrs.get("section"):
            valid["section"] = _("support")
        return valid


class CreateTicketAPIViewSerializer(serializers.ModelSerializer):
    message = serializers.CharField(required=True)
    class Meta:
        model = Ticket
        fields = ("title","section","priority","message")


class AddMessageSerializer(serializers.ModelSerializer):
    class Meta:
        model = TicketMessage
        fields = ("user","ticket","message")

    def validate(self, attrs):
        valid = super().validate(attrs)
        ticket = attrs.get("ticket")

        if ticket.user.id != attrs.get("user").id:
            try: # suspend user :
                attrs.get("user").is_active=False
                attrs.get("user").save()
            except:pass
            raise exceptions.ValidationError({"message":"این تیکت متعلق به شما نیست"})
        
        if(not ticket or ticket.status == "closed"):
            raise exceptions.ValidationError({"message": "تیکت بسته شده است"})
        if attrs.get("message"):
            if any([difflib.SequenceMatcher(None, attrs.get("message"), x.message).ratio()>0.85 for x in  TicketMessage.objects.filter(ticket=ticket)]):
                raise exceptions.ValidationError({"message": "متنی با تشابه تقریبی 85 درصد و بالاتر ، قبلا ارسال شده است."})
            

        return valid


class AddMessageAPIViewSerializer(AddMessageSerializer):
    class Meta(AddMessageSerializer.Meta):
        fields = ("ticket","message")


class SeeCloseTicketSerializer(serializers.ModelSerializer):
    class Meta(AddMessageSerializer.Meta):
        fields = ("ticket",)
        
    
class TicketMessageSerializer(serializers.ModelSerializer):
    class Meta: 
        model = TicketMessage
        fields = '__all__'


class TicketSerializer(serializers.ModelSerializer):
    ticketmessage_set = TicketMessageSerializer(many=True, read_only=True)
    class Meta: 
        model = Ticket
        fields = '__all__'





from django.urls import path
from . import views 
app_name = "ticket"
urlpatterns = [
    path('create_ticket/', views.CreateTicketAPIView.as_view()),
    path('add_message/', views.AddMessageAPIView.as_view()),
    path('close/', views.CloseTicketAPIView.as_view()),
    path('seen/', views.SeeTicketAPIView.as_view()),
    path("get_my_tickets/",views.TicketListAPIView.as_view())
]


from django.contrib import admin
from .models import Ticket, TicketMessage
# Register your models here.
from django.contrib.auth import get_user_model

User = get_user_model()



class UnitInline(admin.TabularInline):
    model = TicketMessage
    extra = 0
    readonly_fields = ("user",)   

    def has_add_permission(self, request, obj=None): return True

    def has_change_permission(self, request, obj=None): return False


@admin.register(Ticket)
class TicketAdmin(admin.ModelAdmin):
    list_display = ('id', 'user', 'title', 'status', 'priority', 'section','created' )
    list_display_links = list_display
    def get_readonly_fields(self, request, obj=None):
        if obj:  # Only apply readonly for existing objects (change view)
            return ["user","title","section","priority","seen_by_user","seen_by_admin","created","updated"]
        return []
    
    
    add_fieldsets = (
        (None, {
            'fields': ('user', 'status',"title","section","priority"),
        }),
    )


    def get_inlines(self, request, obj):
        inlines=[]
        if obj:
            inlines = [
                UnitInline
            ]
        return inlines


    def save_model(self, request, obj, *args, **kwargs):
        if obj.pk :
            for o in obj.ticketmessage_set.all():
                if o.user is None: 
                    o.user = request.user
                    o.save()
        obj.save()
        super().save_model(request, obj, *args, **kwargs)


    def render_change_form(self, request, context, *args, **kwargs):
        
        context.update({'save_text': True})
        return super().render_change_form(request, context, *args, **kwargs)
    

    def save_formset(self, request, form, formset, change):
        instances = formset.save(commit=False)

        for instance in instances:
            # Do something with instance
            if not instance.pk:
                instance.user = request.user
                instance.save()
        formset.save_m2m()




from setuptools import setup
try:
    import pypandoc
    long_description = pypandoc.convert_file('README.md', 'rst')
except(IOError, ImportError):
    long_description = open('README.md').read()
setup(
    name='django-ticket',
    version='2.1.0',
    description='A ticket application for django project',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Hossein Sayyedmousavi',
    author_email='Hossein.Sayyedmousavi@gmail.com',
    packages=['ticket'],
    install_requires=[
                            'django',
                      ],
    url="https://github.com/HosseinSayyedMousavi",
    keywords=[
        "django",
        "database",
        "ticket",
    ],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Environment :: Console",
        "Framework :: Django :: 2.2",
        "Framework :: Django :: 3.2",
        "Framework :: Django :: 4.2",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: BSD License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Topic :: Database",
        "Topic :: System :: Archiving",
        "Topic :: System :: Archiving :: Compression",
    ],
)


import datetime
from datetime import timezone
from django.db import models
from django.contrib.auth import get_user_model

User = get_user_model()


class Ticket(models.Model):
    user         = models.ForeignKey(User, on_delete=models.CASCADE)
    status           = models.CharField(max_length=255, choices=[
        ("pending", "pending"),
        ("answered", "answered"),
        ("closed", "closed")
    ],default="pending")
    
    title           = models.CharField(max_length=255)
    section         = models.CharField(max_length=128,choices=[
        ("management", "management"),
        ("finances", "finances"),
        ("support", "support")
    ])
    priority        = models.CharField(max_length=128,choices=[
        ("low", "low"),
        ("medium", "medium"),
        ("high", "high")
    ],default="low")

    seen_by_user    = models.BooleanField(default=False)
    seen_by_admin   = models.BooleanField(default=False)
    
    created         = models.DateTimeField(auto_now_add=True)
    updated         = models.DateTimeField(auto_now=True)

    def __str__(self):
        return  str(self.id) 


    def save(self, *args, **kwargs):
        
        if self.pk: last_message = self.ticketmessage_set.order_by('-id').first()
        else:last_message = None
        if self.status !="closed" and last_message:
            if last_message.user and last_message.user.is_superuser :
                self.status = "answered"
                self.seen_by_admin = True
                now = datetime.datetime.now(timezone.utc)
                if (now - last_message.created).seconds < 5:
                    self.seen_by_user = False
            else:
                self.status = "pending"

        super(Ticket, self).save(*args, **kwargs)



    

class TicketMessage(models.Model):
    ticket = models.ForeignKey(Ticket, on_delete=models.CASCADE)
    user = models.ForeignKey(User, on_delete=models.CASCADE )
    message = models.TextField()
    created         = models.DateTimeField(auto_now_add=True)
    updated         = models.DateTimeField(auto_now=True )
    def __str__(self):
        return str(self.id)
    
   


from django.dispatch import receiver
from django.db.models.signals import post_save
@receiver(post_save, sender=TicketMessage)
def aftercreate(sender, instance, created, **kwargs):
    if created and instance.ticket and instance.ticket.status !="closed":
        if instance.user and (instance.user.is_superuser or instance.user.is_staff):
            instance.ticket.status = "answered"
        else:
            instance.ticket.status = "pending"
    
    instance.ticket.save()








from rest_framework.response import Response
from .models import Ticket, TicketMessage
from django.contrib.auth import get_user_model
from django.contrib import messages
from rest_framework import generics , status ,permissions
from .serializers import TicketSerializer , CreateTicketSerializer , AddMessageSerializer , SeeCloseTicketSerializer , CreateTicketAPIViewSerializer , AddMessageAPIViewSerializer 
User = get_user_model()


class CreateTicketAPIView(generics.GenericAPIView):
    permission_classes =[permissions.IsAuthenticated]
    serializer_class = CreateTicketAPIViewSerializer

    def post(self,request):
        data = request.data
        data["user"]=request.user.id
        serializer = CreateTicketSerializer(data=data)
        serializer.is_valid(raise_exception=True)
        message = {"message":serializer.validated_data["message"],
                   "user":serializer.validated_data["user"]}
        serializer.validated_data.pop("message")
        ticket = Ticket.objects.create(**serializer.validated_data)
        ticket.ticketmessage_set.create(**message)
        
        return Response({"message": "تیکت با موفقیت ثبت شد."},status=status.HTTP_201_CREATED)


class AddMessageAPIView(generics.GenericAPIView):
    permission_classes =[permissions.IsAuthenticated]
    serializer_class = AddMessageAPIViewSerializer

    def post(self,request):
        data = request.data
        data["user"]=request.user.id
        
        serializer = AddMessageSerializer(data=data)
        serializer.is_valid(raise_exception=True)
        
        TicketMessage.objects.create(**serializer.validated_data)
        
        return Response({"error": 0, "message": "تیکت با موفقیت ثبت شد.", "type": "success"})


class CloseTicketAPIView(generics.GenericAPIView):
    permission_classes =[permissions.IsAuthenticated]
    serializer_class = SeeCloseTicketSerializer
    def patch(self,request):
        data = request.data
        data["user"]=request.user.id
        serializer = self.serializer_class(data=data)
        serializer.is_valid(raise_exception=True)
        ticket = serializer.validated_data["ticket"]
        ticket.status = "closed"
        ticket.save()
        messages.add_message(request, messages.SUCCESS, 'بسته شد')
        return Response({"message": "بسته شد"})


class SeeTicketAPIView(generics.GenericAPIView):
    permission_classes =[permissions.IsAuthenticated]
    serializer_class = SeeCloseTicketSerializer
    def patch(self,request):
        data = request.data
        data["user"]=request.user.id
        serializer = self.serializer_class(data=data)
        serializer.is_valid(raise_exception=True)
        ticket = serializer.validated_data["ticket"]
        ticket.seen_by_user = True
        ticket.save()
        
        return Response({"message": "رویت شد"})


class TicketListAPIView(generics.ListAPIView):
    permission_classes =[permissions.IsAuthenticated]
    serializer_class = TicketSerializer
    def get_queryset(self):
        queryset = Ticket.objects.filter(user = self.request.user)
        return queryset
    
